/*   General   */

a, a:link, a:visited, a:active {
    color: #0069A1; 
    text-decoration: none;
}

a:hover { text-decoration: underline; }

body {
    background-image: url("img/gray_gradient.png");
    background-repeat: repeat-x;
    background-color: white;
    font-family: Verdana, Arial, Sans Serif;
    font-size: 13px;
    margin: 5px 10px;
    color: black;
}

h1, h2 {
    font-size: x-large;
    font-weight: normal;
}

h1 {
    color: #203C66;
    margin: 10px;
}

h3, h4 { font-weight: bold; }

hr {
    size: 0px;
    border-top: 2px solid #e8e8e8;
    margin: 8px 2px;
}

.table {
    padding: 4px;
    margin: 2px;
}

.table-bordered {
    border: 2px solid #e8e8e8;
    -moz-border-radius: 6px;
    -webkit-border-radius: 6px;
}

th {
    background-color: #c0c0c0;
    font-weight: bold;
    vertical-align: top;
}

td {
    vertical-align: top;
    padding: 4px;
}

td.bordered {
    border: 1px solid gray;
}

td.indent { border-left: 4px solid white; }

th, td.heading {
    font-weight: bold;
    margin: 4px 0px;
    padding: 8px;
    background-color: #d8d8d8;
}

td.fieldname {
    background-color: #eeeeee;
    text-align: right;
    padding-right: 10px;
    margin: 2px 0px;
}

td.fieldvalue {
    margin: 2px 0px;
    vertical-align: middle;
}

td.fieldname_error {
    background-color: #ff8888;
    text-align: right;
}

td.fieldvalue_error {
    background-color: #ff8888;
    font-weight: bold;
}

td.navbar {
    border: 0px;
    text-align: center;
    vertical-align: middle;
}

td.friend {
    background-color: #e8e8e8;
    text-align: center;
    vertical-align: middle;
}

.row0 {
    background-color: #d9d9d9;
    text-align: left;
}
.row1 {
    background-color: #eeeeee;
    text-align: left;
}

.highlighted_row0 {
    background-color: #b9d9f9;
    text-align: left;
}
.highlighted_row1 {
    background-color: #ceeefe;
    text-align: left;
}

.row_hd0 { background-color: #cffacf; }

.row_hd1 { background-color: #defade; }

tr.message { background-color: #e0e0e0; }

input[type="text"], select, textarea, .btn {
    border: 1px solid #d8d8d8;
    -moz-border-radius: 5px;
    -webkit-border-radius: 5px;
    padding: 2px 4px;
    font-size: 12px;
}

input[type="button"], input[type="submit"], input.btn {
    margin: 2px 0px;
    background: #d4d0c8;
    margin-right: 0.6em;
    color: #203C66;
    border: 1px solid #a8a8a8;
    font-size: 12px;
    font-weight: normal;
}

/* IE6 doesn't understand [type=XXXX] so we factor this out into its own */
.btn {
    margin: 2px 0px;
    background: #d4d0c8;
    margin-right: 0.6em;
    white-space: nowrap;
    color: #203C66;
    border: 1px solid #a8a8a8;
    font-size: 0.9em;
    font-weight: normal;
    line-height: 2em;
}

.btn:hover {
    background: #edece8;
    text-decoration:none;
}

input[type="button"]:hover, input[type="submit"]:hover, input.btn:hover, .forum_toplinks a:hover {
    background: #edece8;
}

.actionlist {
    display: inline;
    list-style: none;
    margin: 0;
    padding: 0;
}
.actionlist li {
    display: inline;
}


img { border: 0px; }

img.userimg { border: 0px; }

/*   Main Page   */

#news {
    background-color: #dff0ff;
    border: 2px solid #add8e6;
    padding: 10px;
    margin: 4px;
    -moz-border-radius: 10px;
    -webkit-border-radius: 10px;
}

#news h2, #uotd h2, #mainnav h2 {
    margin-top: 0px;
    font-size: 1.2em;
    font-weight: bold;
}

#news h3 {
    color: #666666;
    font-size: 1em;
    margin-bottom: 2px;
}

#news p {
    margin-top: 0px;
}

#uotd {
    background-color: #d3d3d3;
    border: 2px solid #eeeeee;
    padding: 10px;
    margin: 4px;
    -moz-border-radius: 10px;
    -webkit-border-radius: 10px;
}

/*   Forum   */

td.category , tr.subtitle{
    background-color: #dddddd;
    border: 0px;
    font-weight: bold;
    margin: 6px 0px;
}

th {
    font-weight: bold;
    margin: 4px 2px;
    padding: 4px;
}

td.postheader { 
    background-color: #eeeeee;
    height: 24px;
    margin-left: 4px;
    padding-left: 8px; 
    -moz-border-radius: 0px 0px 10px 10px; 
    -webkit-border-radius: 0px 0px 10px 10px; 
}

td.postbody { 
    font-size: 9pt;
    margin-left: 4px;
    background: transparent url('img/75pct_white.png');
}

td.postfooter {
    background-color: #eeeeee; 
    height: 26px; 
    margin-left: 4px; 
    padding-left: 8px; 
    -moz-border-radius: 10px 10px 0px 0px; 
    -webkit-border-radius: 10px 10px 0px 0px; 
}

tr.helpdeskseparator { height: 10pt; }

tr.postseparator {
    background-color: #c8c8c8;
    border: 1px solid #aaaaaa;
    margin: 4px 0px;
}

td.threadline { text-align: left; }

td.numbers {
    text-align: center;
}

td.numbers {
    text-align: left;
}

td.lastpost {
}

.title {
    font-size: 14px;
    font-weight: bold;
}

.text-info, .description {
    font-size: 80%;
    font-weight: normal;
}

.authorcol {
    width: 136px;
    overflow: hidden;
}

div.authorcol { 
    border: 1px solid #c8c8c8; 
    background-color: white; 
    padding: 8px 5px; 
    width: 120px; 
    margin-left: 2px; 
    -moz-border-radius: 10px; 
    -webkit-border-radius: 10px; 
} 

.authorinfo img { 
    border: 1px solid #a8a8a8; 
    margin: 3px 9px; 
} 

.authorcol input { margin: 4px 10px; } 

blockquote.postbody {
    border-left: 2px solid #0089e1;
    background-color: #f5fffa;
    padding: 2px 6px;
    margin: 0px 6px 0px 10px;
    font-style: italic;
} 

#thread {
    width: 100%;
    table-layout: fixed;
    overflow: visible;
}

.forum_toplinks td {
    vertical-align: middle;
}

span.page_title {
    font-size: 24px;
    margin: 20px;
}

p.text-muted, span.note {
    font-weight: normal;
    font-size: 0.9em;
    font-style: italic;
}

span.news_date {
    color: #646464;
    font-size: 0.9em;
}
span.news_title {
    font-weight: bold;
}

span.inboxunread {
    font-weight: bold;
}

span.highlight {
    background-color: #ffffcc;
}

.nobr{
    white-space: nowrap;
}

.code {
    font-family: "Courier New", courier, monospace;
    display: block;
    margin-left: 5em;
    border-left: 3px solid #ccaaaa; 
    padding-left: 1em;
    white-space: nowrap;
    overflow: auto;
}

small {
    font-size: 0.8em;
}

p.text-danger, .error {
    color: #ff0000;
    font-weight: bold;
    font-size: 1.1em;
}

.notice {
    color: #009900;
    font-weight: bold;
    font-size: 1.1em;
}

#preview {
    border: 2px solid #cccccc;
    background-color: #eeeeee;
    margin: 1em;
    padding: 0.2em;
}

#preview .header {
    font-weight: bold;
    font-size: 1.3em;
    border-bottom: 1px solid #cccccc;
}

/*   Server Status Page   */

td.running { background-color: #9aff4f; }

td.notrunning { background-color: #feff6f; }

td.disabled { background-color: #ff4f4f; }

/*   IE hack   */

* html body .code {
    white-space: normal;
}
