<?php

// Plugin definition
$plugin = array(
  'title' => t('Three column 33/34/33'),
  'category' => t('Columns: 3'),
  'icon' => 'threecol_33_34_33.png',
  'theme' => 'panels_threecol_33_34_33',
  'css' => 'threecol_33_34_33.css',
  'panels' => array(
    'left' => t('Left side'),
    'middle' => t('Middle column'),
    'right' => t('Right side')
   ),
);
