
.panel-2col-bricks { 
/*  overflow: hidden;  */
  margin-top: 0;
  padding-top: 0;
}

.panel-2col-bricks .panel-col-top,
.panel-2col-bricks .panel-col-middle,
.panel-2col-bricks .panel-col-bottom { 
  width: 99.9%; 
  clear: both;
}

.panel-2col-bricks .panel-col-top .inside,
.panel-2col-bricks .panel-col-middle .inside {
  margin-bottom: .5em;
}

.panel-2col-bricks .panel-col-first { 
  float: left; 
  width: 50%; 
}
* html .panel-2col-bricks .panel-col-first {
  width: 49.9%;
}

.panel-2col-bricks .panel-col-first .inside { 
  margin: 0 .5em .5em 0;
}

.panel-2col-bricks .panel-col-last { 
  float: left; 
  width: 50%; 
}
* html .panel-2col-bricks .panel-col-last {
  width: 49.9%;
}

.panel-2col-bricks .panel-col-last .inside { 
  margin: 0 0 .5em .5em;
}

.panel-2col-bricks .panel-separator {
  margin: 0 0 1em 0;
}
