<?php

define("DONATION_LINK", "<img src=\"img/donor.png\" alt=\"Project donor\" title=\"Project donor\" />");

// show user a summary of their donations
//
function show_user_donations_private($user) {
}

?>
